<?php
/**
 * unitcalendar.php
 *
 * To generate the unit calendar bar graph. This actually generates the JSON data for the
 * flashchart and not an actual graph. Gets stored in session (id based on divid) so the
 * flashcart SWF player can call another controller function (chart/chart/get) to get the
 * data from session. All flashcart graphs have to use chart/chart/get.
 *
 * @author $Author: dtong $
 * @version $Id: unitcalendar.php,v 1.9 2012/02/24 17:19:38 dtong Exp $
 * @copyright Copyright (c) 2009, Tiller Software Co., Ltd.
*/
class Unitcalendar extends CI_Controller{
	const DIVID = 'unit_calendar';
	// Data for generating a hbar graph. We could have written a class to better handle the data but
	// the old code was very messy and not enough time to make it perfect. So, just us this ad-hoc array data.
	// x - x-axis title text
	// y - y-zis title text
	// bar - array(start_period, end_period, tooltip_text, onclick_jscript)
	private $hbar_data = array('x' => array(), 'y' => array(), 'bar' => array(), 'x_legend' => FALSE);
	private $color_background = '#FFFFF8';

	function __construct() {
		parent::__construct();
        //Everyone should have at least desc read rights
        $this->access_model->checkaccess_any('course_description_read');
        $this->CI = &get_instance();
        $this->load->model('course/course_model'); // model course
        osa_load_lang('desc');
        $this->CI->load->library('cms/osa_chart');
   }

   // The main controller function.
	function process($courseid) {
		if ( !osa_is_int_one($courseid) ) {
			return;
		}
		$data = new stdClass();
		$data->divid = self::DIVID;
		$data->title = '';
		$data->valid_data = FALSE;
		$data->unit_count = 0;
		// $data->color_background = $this->color_background;

		$result = '';
		if ( osa_3p_available() && $this->config->item('cms_3p_blocksched_enabled') ) {
			if ( $obj= $this->cms_3p->load_lib(CMS_3P_LIB_BLOCKSCHED_CALENDAR, CMS_3P_FOLDER_BLOCKSCHED) ) {
				$obj->prepare($courseid, $data, $this->hbar_data);
				if ( $data->valid_data ) {
					// Generate graph using user defined time period(s).
					$result = $this->_generate(FALSE, TRUE, TRUE);
				}
			}
		}
		else {
			$unit = $this->course_model->get_unit_calendar($courseid);
			if (is_array($unit)){
				// Generate graph based on start date and end date.
				$result = $this->_prepare_unit_barchart($unit);
				// To get the course name
				$tmp_unit = &$unit[0];
			  	$data->title =  $tmp_unit->coursename;
			  	$data->valid_data = TRUE;
			  	$data->unit_count = count($unit);
			}
		}
		// Send the unit total to the ajax clipboard. The jscript needs this to figure out
		// how tall the unitcalendar window show be.
		if ( empty($result) || $result == '{}' ) {
			$result = FALSE;
		}
		else {
			$result = TRUE;
		}
		// Default error window size.
		$width = 600;
		$height = 120;
		$chart_width = 0;
		$chart_height = 0;
		if ( $data->unit_count ) {
			$width = 1100;
			$chart_width = 1060;
			// So, each bar is roughly 25px.
			$chart_height = 60 + ($data->unit_count*25);
			$height = $chart_height + 45;
		}
		// Send array to clipboard.
		osa_buffered_output(osa_ajax_clipboard_array(array($result, $width, $height, $chart_width, $chart_height)));
		$this->CI->load->view('course/course_barchart_view.php', $data);
	}

	// Called from javascript to get the unit total so the javascript can modify the window height
	// in real-time.
	function unittotal($courseid) {
		echo $this->course_model->unit_total($courseid);
		exit;
	}

	// Constructs the hbar data for the calendar based on start date and end date.
	// This is business logic only and not dealing with the flashcart directly.
	private function _prepare_unit_barchart($unit_array) {
		$hbars = array();
		$min_year = FALSE;
		$max_year = FALSE;
		$strlen = 0;
		//Let 's loop stratdate and enddate from unit
		$y_labels = array();
		$years_start = array();
		$years_finish = array();

		// Loop thru the units and setup the proper data with each unit and put into the $times array.
		foreach ($unit_array as $unit) {
			$incorrect_date = FALSE;
			$time_s = strtotime($unit->startdate);
			$time_e = strtotime($unit->enddate);
			// In case end time happens before start time. Show a long period plus additional
			// text (code at the bottom of loop ) and hope the user can see the abnormal dates.
			if ( $time_s > $time_e ) {
				$time_e = $time_s + 3600*24*365;
				$incorrect_date = TRUE;
			}

			$temp = array();
			if ( $incorrect_date ) {
				// Add error text to title if start date is greater than the end date.
				$temp['date_error'] = lang('unit_calendar_incorrect_dates');
			}
			$temp['object'] = $unit;
			$temp['start']['date'] = $unit->startdate;
			$temp['start']['time'] = $time_s;
			$temp['start']['year'] = date('Y',$time_s );
			$temp['start']['month'] = date('m',$time_s );
			$temp['start']['day'] = date('d',$time_s );
			$temp['start']['all'] = date('Ymd',$time_s );

			$temp['end']['date'] = $unit->enddate;
			$temp['end']['time'] = $time_e;
			$temp['end']['year'] = date('Y',$time_e );
			$temp['end']['month'] = date('m',$time_e );
			$temp['end']['day'] = date('d',$time_e );
			$temp['end']['all'] = date('Ymd',$time_e );

			$times[] = $temp;
			if ($min_year==FALSE) {
				$min_year = $time_s;
			}
			elseif ($min_year >= $time_s) {
				$min_year = $time_s;
			}

			if ($max_year==FALSE) {
				$max_year = $time_e;
			}
			elseif ($max_year <= $time_e) {
				$max_year = $time_e;
			}

			$years_start[] = $time_s;
			$years_finish[] = $time_e;
		}

		$from_text = lang('unit_calendar_from');
		$to_text = lang('unit_calendar_to');
		// Loop thru the times array (units with massaged data) and create the horizontal bars and y lables.
		foreach ($times as $time) {
			$year_s = $time['start']['year'];
			$year_e = $time['end']['year'];
			$month_s = $time['start']['month'];
			$month_e = $time['end']['month'];
			$day_s = $time['start']['day'];
			$day_e = $time['end']['day'];

			// Get total # of days for the particular month.
			$day_total_s = $this->_days_per_month($month_s,$year_s);
			$day_total_e = $this->_days_per_month($month_e,$year_e);

			$time['start']['diff'] = ($year_s -  date('Y', $min_year)) * 12;
			$time['start']['diff'] += ( 12-date('m', $min_year)+1 ) - (12-$month_s);

			$time['end']['diff'] = ($year_e -  date('Y', $min_year)) * 12;
			$time['end']['diff'] += ( 12-date('m', $min_year)+1 ) - (12-$month_e);

			$start_date = $time['start']['diff'];
			$end_date = $time['end']['diff'];

			if ($day_total_s != '' && $day_total_s != 0) {
				$start_date += (int)$day_s/$day_total_s;
			}

			if ($day_total_e != '' && $day_total_e != 0) {
				$end_date += (int)$day_e/$day_total_e;
			}

			$unit = $time['object'];

			$startdate = osa_date_format($unit->startdate);
			$enddate = osa_date_format($unit->enddate);

			// (issue: 89) Make the first day of the month fall on the month line for the graph
			if ( $start_date - (int) $start_date < 0.04 ) {
				$start_date = (int) $start_date;
			}

			// hbar data
			$newtext = wordwrap($unit->title, 30, "\n", true);
			$newtext = htmlspecialchars_decode($newtext);
			/*
			$tooltip = $newtext . '<br />' . lang('unit_calendar_startdate').': ' .
						  $startdate . ' --> '.lang('unit_calendar_enddate').': '.$enddate;
			*/
			$tooltip = "$newtext ($from_text $startdate $to_text $enddate)";
			$url = base_url() . "unit/unit/read_unit_calendar/".$unit->unitid;
			// (issue: 141) must specific parent div id
			$on_click = "showChartUnitlist('$url','" . lang('unit_calendar_window_title') .
							"',900,800,'unitread','unit_calendar')";
			$tmp_bar = array($start_date, $end_date, $tooltip, $on_click);
			// y label text
			if ( isset($time['date_error']) && $time['date_error'] ) {
				$y_labels[] = $newtext . $time['date_error'];
				// Override the bar color for date error.
				$tmp_bar[4] = '#FF0000';
			}
			else {
				$y_labels[] = $newtext;
			}
			$hbars[] = $tmp_bar;
		}

		// $y_labels = array_reverse($y_labels);
		$max_loop  = ( date('Y', $max_year) -  date('Y', $min_year)) * 12 ;
		$max_loop += ( 12-date('m', $min_year)+1 ) - (12-date('m', $max_year)) ;
		$max_loop += 1; // begin at 0 and end at next last month

		//x_axis
		$months = array(1=>lang('gen_cal_jan'), 2=>lang('gen_cal_feb'), 3=>lang('gen_cal_mar'),
					 		4=>lang('gen_cal_apr'), 5=>lang('gen_cal_may'), 6=>lang('gen_cal_jun'),
					 		7=>lang('gen_cal_jul'), 8=>lang('gen_cal_aug'), 9=>lang('gen_cal_sep'),
							10=>lang('gen_cal_oct'), 11=>lang('gen_cal_nov') ,12=>lang('gen_cal_dec'));
		if ( (int)date('m', $min_year) == 1 ) {
			// (issue: 89) Month in graph gets shifted right when the minimum date falls on Janurary
			$last_year = (int)date('y', $min_year) - 1;
			$last_month = 12;
		}
		else {
			$last_year = (int)date('y', $min_year);
			$last_month = (int)date('m', $min_year) - 1;
		}
		$x_labels = array();
		// Create the x label text and put them in an array
		for ( $i=0; $i<=$max_loop; $i++ ) {
			if ($last_month > 12) {
				$last_month = 1;
				$last_year += 1;
			}
			if ( (int)$last_year < 10 ) {
				$format_year = '0' . $last_year;
			}
			else {
				$format_year = $last_year;
			}
			if ( array_key_exists($last_month, $months) ) {
				$x_labels[] = $months[$last_month] . $format_year;
			}
			$last_month += 1;
		}
		$this->hbar_data['x'] = $x_labels;
		$this->hbar_data['y'] = $y_labels;
		$this->hbar_data['bar'] = $hbars;
		return $this->_generate();
	}

	// Generates the falshchart based on the data structure in $this->hbar_data and
	// put it into the session for use in the chart controller.
	// This is becoming more generic than how it was implemented the first time. It was intended to be generic
	// from the begining but the person who did the orginal code did not understand OOP and reusability
	// despite the requirements were provided to make it generic, difficult to find good OOP programmers...
	//
	// If more bar charts are needed then we should put this function in a separate class and implement a data class
	// to replace the ad-hoc hbar array. Do not copy this code for other bar charts like all bad programmers do,
	// modify it and reuse.
	//
	// $horizontal_threshold - Set to false to turn off vertical x labels.
	//									Uses vertical labels once reaches this threshold.
	// $random_bar_color - True to use, false to use one single predefined color.
	private function _generate($horizontal_threshold=10, $random_bar_color=TRUE, $skip_fontsize_check=FAlSE) {
		$color_tooltip_frame_border = '#000000';
		$color_tooltip_background = '#FFF8DC';
		$color_bar = '#32CD32';
		//$color_background = '#F8F8D8';
		$color_background = $this->color_background;
		$fontsize_ylabel = 12;
		$fontsize_ylabel_small = 11;
		$fontsize_xlabel = 13;
		$fontsize_xlabel_small = 12;
		$fontsize_xlabel_smallest = 11;

		if ( $random_bar_color ) {
			// Need this for random color generation.
			require_once(APPPATH . 'libraries/cms/Osa_chartdata.php');
			$colorobj = new randomColorGenerator();
		}

		$count_data_x = count($this->hbar_data['x']);
		$count_data_y = count($this->hbar_data['y']);
		$count_data_bar = count($this->hbar_data['bar']);
		// Bar label count and bar count should be the same.
		if ( !$this->hbar_data || $count_data_x <= 0 || $count_data_y <= 0 ||
			  $count_data_y != $count_data_bar ) {
			osa_log(__METHOD__, 'Invalid horizontal bar data, bar graph generation aborted.', $this->hbar_data);
			$this->CI->session->unset_userdata('fc_' . self::DIVID);
			return FALSE;
		}
		$chart = new open_flash_chart();
		$chart->set_bg_colour($color_background);

		// Set the x-axis labels
		$x = new x_axis();
		$x->set_offset(FALSE);
		// Add the X Axis Labels to the X Axis
		$x_lbl = new x_axis_labels();
		$x_lbl->set_labels($this->hbar_data['x']);
		$x_lbl->set_size($fontsize_xlabel);
		$x->set_labels($x_lbl);
		if ( !$skip_fontsize_check ) {
			if ( $count_data_x > 10 ) {
				// Too many labels so set smaler font size
				$x_lbl->set_size($fontsize_xlabel_small);
				if ( $count_data_x > 20 ) {
					$x_lbl->set_size($fontsize_xlabel_smallest);
				}
			}
		}
		if ( osa_is_int($horizontal_threshold) && $count_data_x > $horizontal_threshold ) {
			// Setting vertical labels will allow more labels
			$x_lbl->set_vertical();
			/*
			$angle = 30;
			if ( $count_data_x > 20 ) {
				$angle = 50;
			}
			$x_lbl->rotate($angle);
			*/
		}
		$chart->set_x_axis($x);

		// y axis, the lables and horizontal bars
		$y = new y_axis();
		$y->set_offset(TRUE);
		$y->set_range(1, $count_data_y);
		$y_labels = new y_axis_labels();
		if ( $count_data_y > 10 ) {
			$y_labels->set_size($fontsize_ylabel_small);
		}
		else{
			$y_labels->set_size($fontsize_ylabel);
		}
		$y_labels->set_labels(array_reverse($this->hbar_data['y']));
		$y->set_labels($y_labels);
		$chart->set_y_axis($y);
		/*
		// Testing with label color but it didn't work, flashcahrt problem.
		// TODO: Keep this here and revisit this issue later.
		$y_labels = new y_axis_labels();
		$y_labels->set_size($fontsize_ylabel);
		$y_labels->set_colour('red');
		$label_array = array();
		$count = 1;
		foreach (array_reverse($this->hbar_data['y']) as $tmp) {
			$tmp = new y_axis_label($count++, $tmp);
			$tmp->set_colour('blue');
			$tmp->set_size(20);
			$label_array[] = $tmp;
		}
		$y_labels->set_labels($label_array);
		$y->set_labels($y_labels);
		$chart->set_y_axis($y);
		*/

		// Do the horizontal bars from the data structure.
		$hbars = new hbar($color_bar);
		foreach ($this->hbar_data['bar'] as $bar) {
			$val = new hbar_value($bar[0] , $bar[1]);
			$val->set_tooltip($bar[2]);
			$val->on_click($bar[3]);
			if ( isset($bar[4]) && $bar[4] ) {
				$val->set_colour($bar[4]);
			}
			elseif ( $random_bar_color ) {
				$val->set_colour('#' . $colorobj->getColor());
			}
			$hbars->append_value($val);
		}
		$chart->add_element($hbars);

		//Set Tooltip
		$tooltip = new tooltip();
		$tooltip->set_hover();
		$tooltip->set_stroke(1);
		$tooltip->set_colour($color_tooltip_frame_border);
		$tooltip->set_background_colour($color_tooltip_background);
		$chart->set_tooltip($tooltip);

		// Set X legend
		if ( $this->hbar_data['x_legend'] ) {
			$x_legend = new x_legend($this->hbar_data['x_legend']);
			$x_legend->set_style('{font-size:15px;color:#000000}');
			$chart->set_x_legend( $x_legend );
		}

		$result = $chart->toPrettyString();
		$this->CI->session->set_userdata('fc_' . self::DIVID, $result);
		// Return value not really used becuase the chart controller gets the data from the session.
		return $result;
	}

	private function _days_per_month($month, $year) {
		$day = '';
		if ( checkdate($month, 31, $year) ) {
			$day = 31;
		}
		elseif ( checkdate($month, 30, $year) ) {
			$day = 30;
		}
		elseif ( checkdate($month, 29, $year) ) {
			$day = 29;
		}
		else {
			$day = 28;
		}
		return $day;
	}
}
