<?php
/**
 * Cms_captcha.php - provide CAPTCHA (confirmation code) image
 *
 * @author $Author: dtong $
 * @version $Id: Cms_captcha.php,v 1.3 2011/12/30 06:28:20 dtong Exp $
 * @copyright Copyright (c) 2011, Tiller Software Co., Ltd.
*/

Class Cms_captcha {
	// CAPTCHA image width
	private $capt_img_width;
	// CAPTCHA image height
	private $capt_img_height;
	// CAPTCHA image expirattion time before it will be deleted
	private $capt_img_expiration;

	function __construct(){
		$this->CI = & get_instance();
	}

	private function _get_captcha_config(){
		$this->CI->load->helper('captcha');
		$word = osa_gen_randomstr();
		if ( empty($word) ){
			return FALSE;
		}
		$this->capt_img_height = 40;
		$this->capt_img_width = 120;
		$capt_config = array(
		 	'word' => $word,
			'img_path' => $this->get_image_tmppath(),
    		'img_url' => $this->get_image_tmppath(TRUE),
			'font_path' => $this->_get_fontpath(),
    		'img_width' => $this->capt_img_width,
    		'img_height' => $this->capt_img_height,
			'expiration' => $this->capt_img_expiration
		);
		return create_captcha($capt_config);
	}

	/**
	 * Create captcha image
	 * @return string html image of CAPTCHA
	 */
	function create_captcha(){
		$capt_config = $this->_get_captcha_config();
		if ( !is_array($capt_config) ){
			return FALSE;
		}
		$capt_str = $capt_config['word'];
		// set current CAPTCHA string to session
		$this->set_captchakey_session($capt_str);
		return $capt_config['image'];
	}
	/**
	 * verify if the user's guess was correct
	 * @param string $httppost_key
	 * @return boolean
	 */
	function check_valid_key($httppost_key){
		$session_key = $this->CI->session->userdata(CMS_SESSION_CAPTCHA_KEY);
		if ( empty($session_key) || empty($httppost_key) ) {
			return FALSE;
		}
		$session_key = strtolower($session_key);
		$httppost_key = strtolower($httppost_key);
		if ( $httppost_key == $session_key ){
			return TRUE;
		}
		return FALSE;
    }

    function get_image_tmppath($use_url=FALSE){
    	$img_tmppath = $this->CI->config->item('cms_captcha_image_temppath');
    	$tmp_folder = 'tmp/';
    	if ( !$img_tmppath ) {
    		$img_tmppath = $this->CI->config->item('cms_top_image_folder') . $tmp_folder;
    	}
    	if ( $use_url ) {
    		$img_tmppath =  osa_image_toppath() . $tmp_folder;
    	}
    	return $img_tmppath;
    }
    
	private function set_captchakey_session($capt_str){
		if ( empty($capt_str) ) {
			return FALSE;
		}
   	$this->CI->session->set_userdata(CMS_SESSION_CAPTCHA_KEY, $capt_str);
    }

    private function _get_fontpath(){
		$fontpath = $this->CI->config->item('cms_captcha_fontpath');;
		if ( !$fontpath ) {
			$fontpath = $this->CI->config->item('cms_app_include_folder') . '/fonts/FreeUniversal-Bold.ttf';
		}
		return $fontpath;
    }

}